import pandas as pd
import numpy as np
import json

FROM_FILE = "AuD-Inhalt2.xlsx"
OUT_FILE = "aud.json"

COLOR_PALETTE = [
    "rgb(104, 169, 77)",
    "rgb(102, 75, 154)",
    "rgb(41, 171, 226)",
    "rgb(224, 133, 35)",
    "rgb(214, 207, 126)",
    "rgb(239, 65, 35)",
    "rgb(255, 255, 255)",
]

def check_value(val):
    if isinstance(val, str):
        if len(val) == 0:
            return False
    if isinstance(val, float):
        return not np.isnan(val)
    return True

if __name__ == "__main__":
    table = pd.read_excel(FROM_FILE)

    nds = []
    lks = []
    object_groups = []
    name_to_group = {}
    id_map = {}
    node_ids = 0
    group_ids = 0

    # Build ids and object groups
    for idx, row in table.iterrows():
        id_map[row["id"]] = node_ids
        node_ids += 1
        if row["Art"] not in name_to_group:
            name_to_group[row["Art"]] = group_ids
            group = {"id": group_ids, "name": row["Art"], "color": COLOR_PALETTE[group_ids % len(COLOR_PALETTE)]}
            object_groups.append(group)
            group_ids += 1
    
    # Create nodes and links
    for idx, row in table.iterrows():
        node = {'id':id_map[row["id"]], 'name':row["name"], 'description':row["description"], 'type':name_to_group[row["Art"]], 'video': row["Videodatei"]}
        nds.append({k: v for k, v in node.items() if check_value(v)}) # Remove empty attributes
        for link in str(row["Links"]).split(', '):
            if link != "nan": 
                target = id_map[link] if link in id_map else id_map[int(link)]
                lks.append({'source':id_map[row["id"]], 'target':target})

    # Build JSON
    result = {'nodes':nds, 'links':lks, 'objectGroups': object_groups}
    jr = json.dumps(result, ensure_ascii = True)
    
    # Insert HTML breaks instead of line breaks
    jr = jr.replace('\\n','<br>')

    # Save file
    with open(OUT_FILE, "w") as f:
        f.write(jr)