import { ColorData } from "./graph/nodetype";

export { getClientWidth, getWindowInnerHeight, colorToRGB };

/**
 * Returns the maximum width that should be used if displaying elements inside of wordpress.
 * @returns {number}
 */
function getClientWidth(elementId: string): number {
    return document.getElementById(elementId).clientWidth;
}

/**
 * Returns the maximum height that should be used if displaying elements inside of wordpress.
 * @returns {number}
 */
function getWindowInnerHeight(offset = 200): number {
    return window.innerHeight - offset;
}

/**
 * Simple string builder for color data.
 * @param color ColorData object.
 * @returns String describing color in the "rgb(..,..,..)" format.
 */
function colorToRGB(color: ColorData): string {
    // Make it more robust against edge cases
    const r = isNaN(color.r) ? 0 : color.r;
    const g = isNaN(color.g) ? 0 : color.g;
    const b = isNaN(color.b) ? 0 : color.b;

    return "rgb(" + r + "," + g + "," + b + ")";
}
