import React, { useState } from "react";
import { ColorData, NodeType, NodeTypeData } from "../../common/graph/nodetype";
import { colorToRGB } from "../../common/helpers";
import "./nodetypeentry.css";

type NodeTypeEntryProps = {
    nodeType: NodeType;
    onNodeTypeDelete: (id: number[]) => void;
    onNodeTypeDataChange: (
        requests: NodeTypeData[],
        createCheckpoint?: boolean
    ) => void;
    onNodeTypeSelect: (type: NodeType) => void;
    enableDelete: boolean;
    createCheckpoint: (description: string) => void;
};

function NodeTypeEntry({
    nodeType,
    onNodeTypeDataChange,
    onNodeTypeDelete,
    onNodeTypeSelect,
    enableDelete,
    createCheckpoint,
}: NodeTypeEntryProps) {
    const handleDataChange = function <ValueType>(
        key: keyof NodeTypeData,
        value: ValueType
    ) {
        if (!changed) {
            setChanged(true);
        }

        Object.assign(nodeType, { [key]: value });
        onNodeTypeDataChange([nodeType], false);
    };

    const handleColorChange = (part: keyof ColorData, value: string) => {
        // Parse new color
        let colorValue = parseInt(value);
        colorValue = Math.max(colorValue, 0);
        colorValue = Math.min(colorValue, 255);

        const newColor: ColorData = Object.assign({}, nodeType.color, {
            [part]: colorValue,
        });

        handleDataChange("color", newColor);
    };

    const [changed, setChanged] = useState(false);

    const handleBlur = () => {
        if (changed) {
            createCheckpoint(`Modified node type ${nodeType.name}.`);
            setChanged(false);
        }
    };

    return (
        <div className="node-type" onBlur={handleBlur}>
            <h4>Name</h4>
            <input
                className="node-type-name"
                type={"text"}
                value={nodeType.name}
                onChange={(event) =>
                    handleDataChange("name", event.target.value)
                }
            />
            <br />
            <h4>Color</h4>
            <label htmlFor="node-type-color-r">R</label>
            <input
                id="node-type-color-r"
                className="node-type-color"
                type={"number"}
                min={"0"}
                max={"255"}
                value={nodeType.color.r.toString()}
                onChange={(event) => handleColorChange("r", event.target.value)}
            />
            <br />
            <label htmlFor="node-type-color-g">G</label>
            <input
                id="node-type-color-g"
                className="node-type-color"
                type={"number"}
                min={"0"}
                max={"255"}
                value={nodeType.color.g.toString()}
                onChange={(event) => handleColorChange("g", event.target.value)}
            />
            <br />
            <label htmlFor="node-type-color-b">B</label>
            <input
                id="node-type-color-b"
                className="node-type-color"
                type={"number"}
                min={"0"}
                max={"255"}
                value={nodeType.color.b.toString()}
                onChange={(event) => handleColorChange("b", event.target.value)}
            />
            <div
                className={"color-circle"}
                style={{
                    backgroundColor: colorToRGB(nodeType.color),
                }}
            ></div>
            <br />
            <button onClick={() => onNodeTypeSelect(nodeType)}>
                Select nodes
            </button>
            <br />
            {enableDelete && (
                <button onClick={() => onNodeTypeDelete([nodeType.id])}>
                    Delete
                </button>
            )}
        </div>
    );
}

export default NodeTypeEntry;
